
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <asm/uaccess.h>
static const char devname[]="Linux Thingy";

struct file_operations fops;

wait_queue_head_t pwait;

static size_t poll(struct file *fp, struct poll_table_struct *wait)
{
    printk(KERN_INFO"%s : poll() called\n", devname);
    poll_wait(fp, &pwait, wait);
    printk(KERN_INFO"%s : poll() returned\n", devname);
    return 0;
}

static int ioctl(struct inode *ip, struct file *fp, unsigned int cmd, unsigned long arg)
{
    unsigned long pages;
    switch(cmd)
    {
    case 0:
        pages = __get_free_pages(0, 0);
        if(put_user(pages, (unsigned long *)arg))
            return -EFAULT;
        break;
    default:
        return -EINVAL;
    }
    return 0;
}
static int open(struct inode *ip, struct file *fp)
{
    MOD_INC_USE_COUNT;
    return 0;
} 
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
static int close(struct inode *ip, struct file *fp)
{
    MOD_DEC_USE_COUNT;
    return 0;
} 
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   Initialize the module and return 0 if everything is okay.
 *   Return a negative error-code if not.
 */
int __init init_module()
{
    int result;
    memset(&fops, 0x00, sizeof(fops));
    init_waitqueue_head(&pwait);
    fops.owner   = THIS_MODULE;
    fops.ioctl   = ioctl;
    fops.open    = open;
    fops.release = close;
    fops.poll    = poll;
    if((result = register_chrdev(MAJOR_NR, devname, &fops)) < 0)
    {
         printk(KERN_CRIT"%s : Can't register character device with "\
         "major number %d\n", devname, MAJOR_NR);
         return result;
    }
    printk(KERN_INFO"%s : initialization complete\n", devname);
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  Cleanup the module in preparation for removal.
 */
void cleanup_module()
{
    int ret;
    if((ret = unregister_chrdev(MAJOR_NR, devname)) < 0)
    {
        printk(KERN_ALERT"%s : Can't unregister major number %d (%d)\n", devname, MAJOR_NR, ret);
        return;
    } 
    printk(KERN_INFO"%s : module removed\n", devname);
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
